import logging
from typing import List

import requests

from phageai.phageai_auth import PhageAIConnector


class BacteriophageRepository(PhageAIConnector):
    EXPECTED_HTTP_STATUS = 200
    PATH = "YmFjdGVyaW9waGFnZS8="

    def get_record(self, value: str) -> List[dict]:
        """
        Return dict with bacteriophage meta-data
        """

        result = []

        try:
            response = self._make_request(
                path=self._encode(f"{self._decode(self.PATH)}{value}/"), method="get"
            )

            result = response.json()

            if response.status_code == self.EXPECTED_HTTP_STATUS:
                logging.info(f"[PhageAI] Phage get record executed successfully")
            else:
                logging.warning(f'[PhageAI] Exception was raised: "{result}"')
        except requests.exceptions.RequestException as e:
            logging.warning(f'[PhageAI] Exception was raised: "{e}"')

        return result

    def get_top10_similar_phages(self, value: str) -> List[dict]:
        """
        Return list of dicts contained top-10 most similar bacteriophages
        """

        result = []

        try:
            response = self._make_request(
                path=self._encode(
                    f"{self._decode(self.PATH)}{value}{self._decode('L3RvcC0xMC8=')}"
                ),
                method="get",
            )

            result = response.json()

            if response.status_code == self.EXPECTED_HTTP_STATUS:
                logging.info(
                    f"[PhageAI] Phage top-10 similar phages executed successfully"
                )
            else:
                logging.warning(f'[PhageAI] Exception was raised: "{result}"')
        except requests.exceptions.RequestException as e:
            logging.warning(f'[PhageAI] Exception was raised: "{e}"')

        return result



import os

import requests
import logging

from phageai.phageai_auth import PhageAIConnector


class LifeCycleClassifier(PhageAIConnector):
    """
    Bacteriophage life cycle classifier

    All the research and scientific details were published in the paper:
    DOI: 10.1101/2020.07.11.198606
    """

    PATH = "bGlmZWN5Y2xlX3ByZWRpY3Rpb24v"
    EXPECTED_HTTP_STATUS = 201

    def predict(self, fasta_path: str) -> dict:
        """
        Return dict structure with predicted class (label), prediction accuracy, GC% and sequence length
        for passed bacteriophage FASTA file
        """

        result = {}

        if os.path.exists(fasta_path):
            with open(fasta_path, "rb") as fasta:
                try:
                    response = self._make_request(
                        path=self.PATH,
                        method="post",
                        files=[("file", fasta)],
                    )

                    result = response.json()

                    if response.status_code == self.EXPECTED_HTTP_STATUS:
                        logging.info(
                            f"[PhageAI] Life cycle classifier executed successfully"
                        )
                    else:
                        logging.warning(f'[PhageAI] Exception was raised: "{result}"')
                except requests.exceptions.RequestException as e:
                    logging.warning(f'[PhageAI] Exception was raised: "{e}"')
        else:
            logging.warning(
                f'[PhageAI] Exception was raised: "{fasta_path}" doesn\'t exists'
            )

        return result



from setuptools import setup
from glob import glob
from pathlib import Path


CURRENT_DIR = Path(__file__).parent

long_description = (CURRENT_DIR / "README.md").read_text(encoding="utf8")

description = (
    "PhageAI is an AI-driven software platform using advanced Machine Learning"
    " and Natural Language Processing techniques for deeper understanding of"
    " the bacteriophages genomics."
)

# Read requirements and process as list of strings
dependencies = (CURRENT_DIR / "requirements.txt").read_text()
dependencies = list(map(str.strip, filter(None, dependencies.split("\n"))))


version = "0.4.0"

setup(
    name="phageai",
    version=version,
    license="MIT",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="PhageAI S.A.",
    author_email="contact@phage.ai",
    url="https://github.com/phageaisa/phageai",
    download_url=f"https://github.com/phageaisa/phageai/archive/v{version}.tar.gz",
    setup_requires=["setuptools>=50.3.0", "wheel>=0.35.1"],
    install_requires=dependencies,
    packages=[
        "phageai",
        "phageai.repository",
        "phageai.lifecycle",
    ],
    data_files=glob("examples/*/**"),
    include_package_data=True,
    keywords=[
        "bacteriophages",
        "phages",
        "phage therapy",
        "phage research",
        "phage lifecycle",
        "phage taxonomy",
        "virulent phage",
        "temperate phage",
        "chronic phage",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
    ],
    python_requires=">=3.8",
)



import base64
import logging
import uuid
from urllib.parse import urljoin

import requests

logging.basicConfig(level=logging.INFO)


class PhageAIConnector:
    """
    Generic PhageAI API connector
    """

    REQUEST_TIMEOUT = 30

    BASE_URL = "aHR0cHM6Ly9jb3JlLnBoYWdlLmFpL2FwaS92MS9waGFnZWFpLXBhY2thZ2Uv"

    def __init__(self, access_token: str) -> None:
        if not access_token:
            raise ValueError(
                "[PhageAI] Token Error: Please provide correct access token. If you need more information, please check README."
            )
        if self._is_uuid(access_token):
            raise ValueError(
                "[PhageAI] Token Error: We have change our TOS and Policy. Please login to PhageAI Web platform (https://app.phage.ai/) and create new access token."
            )
        self.access_token = access_token
        self.result = {}

    @staticmethod
    def _is_uuid(value):
        try:
            uuid.UUID(value)
            return True
        except ValueError:
            return False

    def _create_auth_header(self) -> dict:
        return {
            "Authorization": f"Bearer {self.access_token}",
        }

    @staticmethod
    def _encode(value: str) -> str:
        return base64.b64encode(value.encode()).decode("utf-8")

    @staticmethod
    def _decode(value: str) -> str:
        return base64.b64decode(value).decode("utf-8")

    def _create_url(self, path) -> str:
        return urljoin(self._decode(self.BASE_URL), self._decode(path))

    @staticmethod
    def _check_status(response):
        http_error_msg = ""
        if isinstance(response.reason, bytes):
            # We attempt to decode utf-8 first because some servers
            # choose to localize their reason strings. If the string
            # isn't utf-8, we fall back to iso-8859-1 for all other
            # encodings. (See PR #3538)
            try:
                reason = response.reason.decode("utf-8")
            except UnicodeDecodeError:
                reason = response.reason.decode("iso-8859-1")
        else:
            reason = response.reason

        if 400 <= response.status_code < 500:
            http_error_msg = f"[PhageAI] Connection Client Error: Response Status Code - {response.status_code} Reason: {reason}"
        elif 500 <= response.status_code < 600:
            http_error_msg = f"[PhageAI] Connection Server Error: Response Status Code - {response.status_code} Reason: {reason}"

        if http_error_msg:
            raise requests.HTTPError(http_error_msg, response=response)

    def _make_request(self, path: str, method: str, **kwargs) -> requests.Response:
        """
        Generic PhageAI API request method
        """

        headers = self._create_auth_header()

        logging.info(f"[PhageAI] Method: {method}")

        response = getattr(requests, method)(
            url=self._create_url(path),
            headers=headers,
            timeout=self.REQUEST_TIMEOUT,
            **kwargs,
        )

        self._check_status(response)

        return response
